/**
 * @file /include/ecl/containers/push_and_pop/push_and_pop_dynamic.hpp
 *
 * @brief A simple fifo implementation.
 *
 * Internally it uses ecl::Array for various error handling and other
 * routines.
 *
 * @date August 2010
 **/
/*****************************************************************************
** Ifdefs
*****************************************************************************/

#ifndef ECL_CONTAINERS_PUSH_AND_POP_DYNAMIC_HPP_
#define ECL_CONTAINERS_PUSH_AND_POP_DYNAMIC_HPP_

/*****************************************************************************
** Includes
*****************************************************************************/

#include <ecl/config/macros.hpp>
#include "push_and_pop_fixed.hpp"

/*****************************************************************************
** Namespaces
*****************************************************************************/

namespace ecl {

/*****************************************************************************
** Interface
*****************************************************************************/
/**
 * @brief
 * Surpport fixed array and push and pack operation
 * HOWEVER this is now experimental version.
 * Please do not use this container until we complete review
 *
 *
 * <b>Usage</b>:
 *
 *
 * @code
 *
 *
 * @code
 *
 *
 * @endcode
 *
 * @sa ecl::Array.
 */


template<typename T>
class ECL_PUBLIC PushAndPop<T,DynamicStorage>
{
public:
	PushAndPop() :
	size_fifo(0),
	leader(0),
	follower(0)
	{
	}

	PushAndPop( const unsigned int length ) ecl_assert_throw_decl(StandardException)
	: size_fifo(length+1),
		leader(0),
		follower(0)
	{
		ecl_assert_throw( (size_fifo>0), StandardException(LOC, OutOfRangeError, "SimpleFIFO start with zero size buffer"));
		data.resize( size_fifo );
	}

	PushAndPop( const unsigned int length, const T & d ) ecl_assert_throw_decl(StandardException) :
	size_fifo(length+1),
	leader(0),
	follower(0)
	{
		ecl_assert_throw( (size_fifo>0), StandardException(LOC, OutOfRangeError, "SimpleFIFO start with zero size buffer"));
		data.resize( size_fifo );
		fill( d );
	}
	virtual ~PushAndPop() {}

	T & operator[] (int idx)
	{
		return data[ ((follower+idx)%size_fifo) ];
	}

	const T & operator[] (int idx) const
	{
		return data[ ((follower+idx)%size_fifo) ];
	}


	void operator() (const PushAndPop<T,0> & otherOne )
	{
		leader = otherOne.leader;
		follower = otherOne.follower;
		for( int i=0; i<size_fifo; i++ )
		{
			data[i] = otherOne.data[i];
		}
	}


	void push_back( const T & datum )
	{
		data[ leader++ ] = datum;
		leader %= size_fifo;
		if( leader == follower )
		{
			follower++;
			follower %= size_fifo;
			// it just rolls over
		}
	}

	T pop_front()
	{
		ecl_assert_throw( (follower != leader), StandardException(LOC, OutOfRangeError, "PushAndPop follower==leader"));
		T value = data[follower++];
		follower %= size_fifo;
		return value;
	}

	void fill( const T & d )
	{
		for( unsigned int i=0; i<size_fifo; i++ ) data[i] = d;
	}

	void resize( unsigned int length )
	{
		size_fifo = length;
		ecl_assert_throw( (size_fifo>0), StandardException(LOC, OutOfRangeError, "SimpleFIFO start with zero size buffer"));
		data.resize( size_fifo );
	}

	unsigned int asize()
	{
		return size_fifo;
	}

	unsigned int size() const
	{
		if( leader > follower ) return leader - follower;
		else if( leader < follower ) return size_fifo-follower+leader;
		return 0;
	}

	void clear()
	{
		follower = 0;
		leader = 0;
	}

//protected:
	ecl::Array <T>data;
	unsigned int size_fifo;
	int leader;
	int follower;
};



} // namespace ecl

#endif /* ECL_CONTAINERS_PUSH_AND_POP_DYNAMIC_HPP_ */
